package springmvc.repo;

import java.util.List;

import javax.persistence.EntityManager;
import javax.persistence.criteria.CriteriaBuilder;
import javax.persistence.criteria.CriteriaQuery;
import javax.persistence.criteria.Root;

import springmvc.domain.Member;
import springmvc.util.GenericDAO;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Repository;
import org.springframework.transaction.annotation.Transactional;

@Repository
@Transactional
public class MemberDaoImpl extends GenericDAO<Member, Long> implements MemberDao
{
	
	//default constructor start when object created
    public MemberDaoImpl() {
		super(Member.class);
	}



    public Member findById(Long id)
    {
        return entityManager.find(Member.class, id);
    }

    public Member findByEmail(String email)
    {
        CriteriaBuilder builder = entityManager.getCriteriaBuilder();
        CriteriaQuery<Member> criteria = builder.createQuery(Member.class);
        Root<Member> member = criteria.from(Member.class);

        /*
         * Swap criteria statements if you would like to try out type-safe criteria queries, a new
         * feature in JPA 2.0 criteria.select(member).orderBy(cb.asc(member.get(Member_.name)));
         */

        criteria.select(member).where(builder.equal(member.get("email"), email));
        return entityManager.createQuery(criteria).getSingleResult();
    }

    public List<Member> findAllOrderedByName()
    {
        CriteriaBuilder cb = entityManager.getCriteriaBuilder();
        CriteriaQuery<Member> criteria = cb.createQuery(Member.class);
        Root<Member> member = criteria.from(Member.class);

        /*
         * Swap criteria statements if you would like to try out type-safe criteria queries, a new
         * feature in JPA 2.0 criteria.select(member).orderBy(cb.asc(member.get(Member_.name)));
         */

        criteria.select(member).orderBy(cb.asc(member.get("name")));
        return entityManager.createQuery(criteria).getResultList();
    }

    
    //register first function persist
    @Override
    public void register(Member member)
    {
    	entityManager.persist(member);
        return;
    }
}
